# -*- coding: utf-8 -*-
from django.contrib import admin
from .models import Channel


@admin.register(Channel)
class ChannelAdmin(admin.ModelAdmin):
    list_display = ["name", "level", "is_active"]



# -*- coding: utf-8 -*-
import json

from django.contrib import auth
from django.http import JsonResponse, HttpResponseForbidden, JsonResponse
from django.views.decorators.csrf import csrf_exempt
from django.middleware.csrf import get_token
from django.views.decorators.csrf import csrf_exempt

from .token import connection_token, channel_token
from .models import Channel


@csrf_exempt
def private_channel_subscription(request):
    if not request.user.is_authenticated:
        return HttpResponseForbidden()
    json_data = json.loads(request.body)
    # print("PRIVATE SUB for", request.user.username, json_data)
    user_chans = Channel.objects.for_user(request.user).values("name")
    user_chans_names = []
    for chan in user_chans:
        user_chans_names.append(chan["name"])
    authorized_chans = []
    for channel in json_data["channels"]:
        # print("Checking auth for chan", channel, channel in user_chans_names)
        if channel in user_chans_names:
            chan = {
                "channel": channel,
                "token": channel_token(json_data["client"], channel),
            }
            authorized_chans.append(chan)
    # print({"channels": authorized_chans})
    return JsonResponse({"channels": authorized_chans})


def _get_response(request):
    channels = Channel.objects.for_user(request.user).values("name", "level")
    return JsonResponse(
        {
            "csrf_token": get_token(request),
            "ws_token": connection_token(request.user),
            "channels": list(channels),
        }
    )


@csrf_exempt
def login_and_get_tokens(request):
    print("Login view", request.method)
    if request.user.is_authenticated:
        return _get_response(request)
    if request.method == "POST":
        # print("POST", request.body)
        json_data = json.loads(request.body)
        username = json_data["username"]
        password = json_data["password"]
        # print("Authenticate", username, password)
        user = auth.authenticate(username=username, password=password)
        if user is not None:
            auth.login(request, user)
            return _get_response(request)
    return HttpResponseForbidden()


def logout(request):
    if request.user.is_authenticated:
        auth.logout(request)
        return JsonResponse({"response": "ok"})
    return HttpResponseForbidden()


@csrf_exempt
def get_connection_token(request):
    if not request.user.is_authenticated:
        return HttpResponseForbidden()
    return _get_response(request)



import jwt
import time
from .conf import CENTRIFUGO_HMAC_KEY


def connection_token(user):
    if CENTRIFUGO_HMAC_KEY is None:
        raise ValueError("Provide a CENTRIFUGO_HMAC_KEY in settings")
    claims = {"sub": user.username, "exp": int(time.time()) + 24 * 60 * 60}
    token = jwt.encode(claims, CENTRIFUGO_HMAC_KEY, algorithm="HS256")
    return token


def channel_token(client, channel):
    if CENTRIFUGO_HMAC_KEY is None:
        raise ValueError("Provide a CENTRIFUGO_HMAC_KEY in settings")
    claims = {
        "client": client,
        "channel": channel,
        "exp": int(time.time()) + 24 * 60 * 60,
    }
    token = jwt.encode(claims, CENTRIFUGO_HMAC_KEY, algorithm="HS256")
    return token



# -*- coding: utf-8 -*-

from cent import Client, CentException
from .conf import SITE_NAME, CENTRIFUGO_HOST, CENTRIFUGO_PORT, CENTRIFUGO_API_KEY


def publish(channel, *args, event_class=None, data=None, bucket=None, site=SITE_NAME):
    message = None
    if len(args) == 1:
        message = args[0]
    if message is None and data is None:
        raise ValueError("Provide either a message or data argument")
    cent_url = CENTRIFUGO_HOST
    if CENTRIFUGO_PORT is not None:
        cent_url += ":" + str(CENTRIFUGO_PORT)
    client = Client(cent_url, api_key=CENTRIFUGO_API_KEY, timeout=1)
    payload = {"channel": channel, "site": site}
    if message is not None:
        payload["message"] = message
    if data is not None:
        payload["data"] = data
    if event_class is not None:
        payload["event_class"] = event_class
    if bucket is not None:
        payload["bucket"] = bucket
    err = None
    try:
        print(f"Publishing to {cent_url} {payload}")
        client.publish(channel, payload)
    except CentException as e:
        err = str(e)
    except Exception as e:
        raise e
    return err



from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    operations = [
        migrations.CreateModel(
            name='Channel',
            fields=[
                ('id', models.BigAutoField(auto_created=True,
                                           primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(help_text='Use $ to prefix non public channels: ex: $private_chan',
                                          max_length=120, unique=True, verbose_name='Name')),
                ('level', models.CharField(choices=[('public', 'Public'), ('users', 'Users'), ('groups', 'Groups'), (
                    'staff', 'Staff'), ('superuser', 'Superuser')], max_length=20, verbose_name='Authorized for')),
                ('is_active', models.BooleanField(
                    default=True, verbose_name='Active')),
                ('groups', models.ManyToManyField(
                    blank=True, to='auth.Group', verbose_name='Groups')),
            ],
            options={
                'verbose_name': 'Channel',
                'verbose_name_plural': 'Channels',
            },
        ),
    ]



# -*- coding: utf-8 -*-
from django.urls import path
from .views import (
    login_and_get_tokens,
    logout,
    get_connection_token,
    private_channel_subscription,
)

urlpatterns = [
    path("login/", login_and_get_tokens, name="instant-login"),
    path("logout/", logout, name="instant-logout"),
    path("get_token/", get_connection_token, name="instant-get-token"),
    path("subscribe/", private_channel_subscription, name="instant-subscribe"),
]



# -*- coding: utf-8 -*-

def ensure_channel_is_private(chan):
    name = chan
    if ":" in name:
        names = name.split(":")
        prefix = names[0]
        suffix = names[1]
        if suffix.startswith("$") is False:
            return prefix + ":$" + suffix
    else:
        if chan.startswith("$") is False:
            return "$" + chan
    return chan



from django.conf import settings


CENTRIFUGO_API_KEY = getattr(settings, "CENTRIFUGO_API_KEY", None)
CENTRIFUGO_HMAC_KEY = getattr(settings, "CENTRIFUGO_HMAC_KEY", None)
CENTRIFUGO_HOST = getattr(settings, "CENTRIFUGO_HOST", "http://localhost")
CENTRIFUGO_PORT = getattr(settings, "CENTRIFUGO_PORT", 8000)

SITE_SLUG = getattr(settings, "SITE_SLUG", "site")
SITE_NAME = getattr(settings, "SITE_NAME", "Site")



# -*- coding: utf-8 -*-
from django.db import models
from django.contrib.auth.models import Group
from django.utils.translation import ugettext_lazy as _
from .init import ensure_channel_is_private


LEVELS = (
    ("public", "Public"),
    ("users", "Users"),
    ("groups", "Groups"),
    ("staff", "Staff"),
    ("superuser", "Superuser"),
)


class ChannelManager(models.Manager):
    def for_user(self, user):
        filter_from = ["public", "users"]
        if user.is_superuser:
            filter_from.append("superuser")
        if user.is_staff:
            filter_from.append("staff")
        chans = Channel.objects.filter(
            is_active=True, groups__in=user.groups.all()
        ) | Channel.objects.filter(level__in=filter_from, is_active=True)
        return chans


class Channel(models.Model):
    global LEVELS
    name = models.CharField(
        max_length=120,
        verbose_name=_("Name"),
        unique=True,
        help_text=_(
            "Use $ to prefix non public channels: " "ex: $private_chan"),
    )
    level = models.CharField(
        max_length=20, choices=LEVELS, verbose_name=_("Authorized for")
    )
    is_active = models.BooleanField(default=True, verbose_name=_("Active"))
    groups = models.ManyToManyField(
        Group, blank=True, verbose_name=_("Groups"))
    objects = ChannelManager()

    class Meta:
        verbose_name = _("Channel")
        verbose_name_plural = _("Channels")

    def __str__(self):
        return f"{self.name} ({self.level})"

    def save(self, *args, **kwargs):
        # check the channel name
        if self.level != "public":
            if self.name.startswith("$") is False:
                self.name = ensure_channel_is_private(self.name)
        return super(Channel, self).save(*args, **kwargs)
